﻿using GisSharpBlog.NetTopologySuite.Geometries;
using GisSharpBlog.NetTopologySuite.IO;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ThinkGeo.MapSuite.Core;

namespace GetAreaTest
{
    public static class FeatureCollectionUtility
    {
        /// <summary>
        /// A convenience method for reading a FeatureCollection's
        /// feature properties into a list of string dictionaries where the list
        /// elements are the rows, each being a key/value pair of column name and value.
        /// </summary>
        /// <param name="featurecollectionGeoJson">the FeatureCollection to read</param>
        /// <param name="ensureColumns">if true, ID, Name and Area (sq meters) columns will
        /// be ensured (added and populated) if needed, to each dictionary</param>
        /// <returns>null if the geojson is not a FeatureCollection</returns>
        public static List<Dictionary<String, String>> GetFeatureProperties(string featurecollectionGeoJson, bool ensureColumns = false)
        {
            Collection<Feature> fs = new Collection<Feature>();

            string geojson = featurecollectionGeoJson;
            var data = JObject.Parse(geojson);
            if (data["features"].IsNullOrEmpty()) return null;
            List<Dictionary<String, String>> table = new List<Dictionary<String, String>>();
            HashSet<string> columnnames = new HashSet<string>();

            //get the list of columns present and the number of rows
            foreach (var feature in (JArray)data["features"])
            {
                var properties = feature["properties"];
                if (properties.IsNullOrEmpty()) continue;
                foreach (JProperty property in properties)
                {
                    if (!columnnames.Contains(property.Name))
                        columnnames.Add(property.Name);
                }
            }

            //now copy and augment properties
            int rowcount = 0;
            foreach (var feature in (JArray)data["features"])
            {
                rowcount = rowcount + 1;
                Dictionary<String, String> row = new Dictionary<String, String>();
                var properties = feature["properties"];
                if (properties.IsNullOrEmpty()) continue;
                foreach (var columnname in columnnames)
                {
                    if (properties[columnname].IsNullOrEmpty()) continue;
                    row.Add(columnname, properties[columnname].GetValue<string>());
                }

                if (ensureColumns)
                {
                    //make sure we have an ID attribute
                    if (!row.ContainsKey("ID"))
                    {
                        row.Add("ID", rowcount.ToString());
                    }

                    //make sure we have a Name attribute
                    if (!row.ContainsKey("Name"))
                    {
                        row.Add("Name", row["ID"]);
                    }

                    //add 'Area' column in units of square meters
                    ThinkGeo.MapSuite.Core.Feature gfeature = ThinkGeo.MapSuite.Core.Feature.CreateFeatureFromGeoJson(feature.ToString());
                    fs.Add(gfeature);
                    double area = GetUtmAreaMeters(4326, gfeature);

                    if (!row.ContainsKey("Area"))
                        row.Add("Area", area.ToString());
                    else
                        row["Area"] = area.ToString();
                }

                table.Add(row);
            }

            return table;
        }

        /// <summary>
        /// Use Thinkgeo GetArea() to calc the area of a feature's geometry
        /// </summary>
        /// <param name="sourceDataEPSGId"></param>
        /// <param name="sourceFeature"></param>
        /// <returns></returns>
        public static double GetUtmAreaMeters(int sourceDataEPSGId, Feature sourceFeature)
        {
            BaseShape sourceShape = sourceFeature.GetShape();
            PointShape centerShape = sourceShape.GetCenterPoint();
            int utmzoneEpsg = Convert.ToInt32(32700.0 - Math.Round((45.0 + centerShape.Y) / 90.0) * 100.0 + Math.Round((183.0 + centerShape.X) / 6.0));
            Proj4Projection projection = new Proj4Projection();
            projection.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(sourceDataEPSGId);
            projection.ExternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(utmzoneEpsg);
            projection.Open();
            MultipolygonShape projectedShape = projection.ConvertToExternalProjection(sourceShape) as MultipolygonShape;
            var wkb = projectedShape.GetWellKnownBinary();
            var wkbReader = new WKBReader();
            var geometry = wkbReader.Read(wkb);

            return geometry.Area;
        }
    }
}
